<?php

/**
 * ElexShipEngine class.
 *
 * @extends WC_Shipping_Method
 */
class ElexShipEngine extends WC_Shipping_Method {

	private $domestic = array( 'US' );
	private $found_rates;
	private $carrier_list;
   
	/**
	 * Constructor
	 */
	public function __construct() {
		$this->id = ELEX_SHIPENGINE_ID;
		$this->method_title = __( 'ShipEngine (BASIC)', 'wf-shipengine' );
		$this->method_description = __( 'The <strong>ShipEngine.com</strong> plugin obtains rates dynamically from the ShipEngine.com API during cart/checkout.', 'wf-shipengine' );
		$this->services = include( 'data-wf-services.php' );
		$this->rates_endpoint = 'https://api.shipengine.com/v1/rates';
		$this->set_carrier_list();
		$this->init();
	}

	/**
	 * Init function.
	 *
	 * Access public
	 * Return void
	 */
	private function init() {
		// Load the settings.
		$this->init_form_fields();
		$this->init_settings();

		// Define user set variables
		$this->enabled              = isset( $this->settings['enabled'] ) ? $this->settings['enabled'] : $this->enabled;
		$this->title                = ! empty( $this->settings['title'] ) ? $this->settings['title'] : $this->method_title;
		$this->zip                  = isset( $this->settings['zip'] ) ? $this->settings['zip'] : '';
	   $this->resid = isset( $this->settings['show_rates'] ) && 'residential' == $this->settings['show_rates'] ? 'yes' : 'no';
		$this->availability = isset( $this->settings['availability'] ) ? $this->settings['availability'] : 'all';
		$this->countries = isset( $this->settings['countries'] ) ? $this->settings['countries'] : array();
		$this->senderName = isset( $this->settings['name'] ) ? $this->settings['name'] : '';
		$this->senderCompanyName = isset( $this->settings['company'] ) ? $this->settings['company'] : '';
		$this->senderAddressLine1 = isset( $this->settings['street1'] ) ? $this->settings['street1'] : '';
		$this->senderAddressLine2 = isset( $this->settings['street2'] ) ? $this->settings['street2'] : '';
		$this->senderCity = isset( $this->settings['city'] ) ? $this->settings['city'] : '';
		$this->senderState = isset( $this->settings['state'] ) ? $this->settings['state'] : '';
		//adding country
		$this->senderCountry = isset( $this->settings['country'] ) ? $this->settings['country'] : '';
		$this->senderEmail = isset( $this->settings['email'] ) ? $this->settings['email'] : '';
		$this->senderPhone = isset( $this->settings['phone'] ) ? $this->settings['phone'] : '';
		$this->api_key                      = isset( $this->settings['api_key'] ) ? $this->settings['api_key'] : '';
		$this->custom_services              = isset( $this->settings['services'] ) ? $this->settings['services'] : array();
		$this->offer_rates                  = isset( $this->settings['offer_rates'] ) ? $this->settings['offer_rates'] : 'all';
		$this->fallback                     = ! empty( $this->settings['fallback'] ) ? $this->settings['fallback'] : '';
		$this->mediamail_restriction        = isset( $this->settings['mediamail_restriction'] ) ? $this->settings['mediamail_restriction'] : array();
		$this->mediamail_restriction        = array_filter( (array) $this->mediamail_restriction );
		$this->enable_standard_services     = true;
		$this->senderState = isset( $this->settings['state'] ) ? $this->settings['state'] : '';
		$this->carrier_id  = isset($this->settings['shipengine_carrier_id']) ? $this->settings['shipengine_carrier_id'] : array(); 
		$this->debug                  = isset( $this->settings['debug_mode'] ) && 'yes' == $this->settings['debug_mode'] ? true : false;
		$this->api_mode               = isset( $this->settings['api_mode'] ) ? $this->settings['api_mode'] : 'Live';
		$this->carrier = ( isset( $this->settings['shipengine_carrier'] ) && ! empty( $this->settings['shipengine_carrier'] ) ) ? $this->settings['shipengine_carrier'] : array();
		add_action( 'woocommerce_update_options_shipping_' . $this->id, array( $this, 'process_admin_options' ) );
		add_action( 'woocommerce_update_options_shipping_' . $this->id, array( $this, 'clear_transients' ) );
	}

	private function set_carrier_list() {
		foreach ( array_keys( $this->services ) as $key => $carrier ) {
			$carrier_list[ $carrier ] = $carrier;
		}
		$this->carrier_list = $carrier_list;
	}
	/**
	 * Environment check function.
	 *
	 * Access public
	 * Return void
	 */
	private function environment_check() {
		global $woocommerce;

		$admin_page = version_compare( WOOCOMMERCE_VERSION, '2.1', '>=' ) ? 'wc-settings' : 'woocommerce_settings';

		if ( ! $this->zip && 'yes' == $this->enabled ) {
			echo '<div class="error">
                <p>' . esc_attr( 'ShipEngine.com is enabled, but the zip code has not been set.', 'wf-shipengine' ) . '</p>
            </div>';
		}

		$error_message = '';

		// Check for ShipEngine.com APIKEY
		if ( ! $this->api_key && 'yes' == $this->enabled ) {
			$error_message .= '<p>' . __( 'ShipEngine.com is enabled, but the ShipEngine.com API KEY has not been set.', 'wf-shipengine' ) . '</p>';
		}
		$allow_html = array(
			'p' => array(
				'class' => array(),
			),
		);
		if ( '' == ! $error_message ) {
			echo '<div class="error">';
			echo wp_kses( $error_message, $allow_html );
			echo '</div>';
		}
	}

	/**
	 * Admin_options function.
	 *
	 * Access public
	 * Return void
	 */
	public function admin_options() {
		// Check users environment supports this method
		$this->environment_check();

		// Show settings
		parent::admin_options();
	}

	/**
	 * Generate_services_html function.
	 */
	public function generate_services_html() {
		ob_start();
		include( 'html-wf-services.php' );
		return ob_get_clean();
	}

	/**
	 * Validate_services_field function.
	 *
	 * Access public
	 * Param mixed $key
	 * Return void
	 */
	public function validate_services_field( $key ) {
		$services           = array();
		$posted_services    = isset( $_POST['shipengine_services'] ) && isset( $_POST['eh-sp-nonce'] ) && wp_verify_nonce( wp_kses_post( $_POST['eh-sp-nonce'] )  , 'eh_shipengine_nonce' ) ? array_map( 'wp_kses_allowed_html', wp_unslash( $_POST['shipengine_services'] ) ) : array(); 
		foreach ( $posted_services as $code => $settings ) {

			foreach ( $this->services[ $code ]['services'] as $key => $name ) {

				$services[ $code ][ $key ]['enabled']               = isset( $settings[ $key ]['enabled'] ) ? true : false;
				$services[ $code ][ $key ]['order']                 = wc_clean( $settings[ $key ]['order'] );
			
			}
		}
		$this->custom_services = $services; 
		return $services;
	}

	/**
	 * Clear transients function.
	 *
	 * Access public
	 * Return void
	 */
	public function clear_transients() {
		global $wpdb;

		$wpdb->query( "DELETE FROM `$wpdb->options` WHERE `option_name` LIKE ('_transient_shipengine_quote_%') OR `option_name` LIKE ('_transient_timeout_shipengine_quote_%')" );
	}
	
	public function generate_activate_box_html() {
		ob_start();
		$plugin_name = 'shipengine';
		include( 'market.php' );
		return ob_get_clean();
	}

	public function generate_shipengine_tabs_html() {

			$current_tab = ( ! empty( $_GET['subtab'] ) ) ? sanitize_text_field( $_GET['subtab'] ) : 'general';
				echo '
                <div class="wrap">
                    <script>
                    jQuery(function($){
                    show_selected_tab($(".tab_general"),"general");
                    $(".tab_general").on("click",function(){
                                                return show_selected_tab($(this),"general");
                                        });
                    $(".tab_rates").on("click",function(){
                                                return show_selected_tab($(this),"rates");
                                        });
                    $(".tab_gopremium").on("click",function(){                                                
                                                return show_selected_tab($(this),"gopremium");
                                        });
                    function show_selected_tab($element,$tab)
                    {
                        $(".nav-tab").removeClass("nav-tab-active");
                        $element.addClass("nav-tab-active");
                        $(".general_tab_field").closest("tr,h3").hide();
                        $(".general_tab_field").next("p").hide();
                                         
                        $(".rates_tab_field").closest("tr,h3").hide();
                        $(".rates_tab_field").next("p").hide();

                        $(".label_tab_field").closest("tr,h3").hide();
                        $(".label_tab_field").next("p").hide();

                        $(".package_tab_field").closest("tr,h3").hide();
                        $(".package_tab_field").next("p").hide();

                        $(".gopremium_tab_field").closest("tr,h3").hide();
                        $(".gopremium_tab_field").next("p").hide();
                        if($tab=="gopremium")
                        {   
                            $(".marketing_content").show();
                        }else{
                            $(".marketing_content").hide();
                        }
                        $("#woocommerce_wf_shipengine_availability").trigger("change");
                        $("."+$tab+"_tab_field").closest("tr,h3").show();
                        $("."+$tab+"_tab_field").next("p").show();
                        
                         if($tab=="rates")
                        {
                        	if(document.getElementById("woocommerce_elex_shipengine_id_availability").value=="specific")
							{
					            $("#woocommerce_elex_shipengine_id_countries").closest("tr").show();
							}	
							else
							{
					            $("#woocommerce_elex_shipengine_id_countries").closest("tr").hide();
							}
                        }
                        else
                        {
                        	$("#woocommerce_elex_shipengine_id_countries").closest("tr").hide();
                        }

                        if($tab=="gopremium")
                        {
                            $(".woocommerce-save-button").hide();
                        }else
                        {
                            $(".woocommerce-save-button").show();
                        }
                        return false;
                    }   

                    });
                    </script>
                    <style>
                    .wrap {
                                min-height: 800px;
                            }
                    a.nav-tab{
                                cursor: default;
                    }
                    </style>
                    <hr class="wp-header-end">';
					$tabs = array(
						'general' => __( "General <span style='vertical-align: super;color:green;font-size:12px'></span>", 'wf-shipengine' ),
						'rates' => __( "Rates & Services <span style='vertical-align: super;color:green;font-size:12px'></span>", 'wf-shipengine' ),
						'labels' => __( "Label Generation <span style='vertical-align: super;color:green;font-size:12px'>[Premium]</span>", 'wf-shipengine' ),
						'packing' => __( "Packaging <span style='vertical-align: super;color:green;font-size:12px'>[Premium]</span>", 'wf-shipengine' ),
						'gopremium' => __( "Go Premium! <span style='vertical-align: super;color:green;font-size:12px'></span>", 'wf-shipengine' ),
					);
					$html = '<h2 class="nav-tab-wrapper">';
					foreach ( $tabs as $stab => $name ) {
						$class = ( $stab == $current_tab ) ? 'nav-tab-active' : '';
						$style = ( $stab == $current_tab ) ? 'border-bottom: 1px solid transparent !important;' : '';
						$style = ( 'gopremium' == $stab ) ? $style . 'color:red; !important;' : '';
						$html .= '<a style="text-decoration:none !important;' . $style . '" class="nav-tab ' . $class . ' tab_' . $stab . '" >' . $name . '</a>';
					}
					$html .= '</h2>';
					echo wp_kses_post( $html );

	}
	
	
	/**
	 * Init_form_fields function.
	 *
	 * Access public
	 * Return void
	 */
	public function init_form_fields() {
		global $woocommerce;

		$classes = get_terms( 'product_shipping_class', array( 'hide_empty' => '0' ) );

		$classes = ( $classes ) ? $classes : array();
		
		$shipping_classes = array();
	
		foreach ( $classes as $class ) {
			$shipping_classes[ $class->term_id ] = $class->name;
		}

		if ( ELEX_SHIPENGINE_ADV_DEBUG_MODE == 'on' ) { // Test mode is only for development purpose.
			$api_mode_options = array(
				'Live' => __( 'Live', 'wf-shipengine' ),
				'Test' => __( 'Test', 'wf-shipengine' ),
			);
		} else {
			$api_mode_options = array(
				'Live' => __( 'Live', 'wf-shipengine' ),
			);
		}
		
		$this->form_fields = array(
			
			'shipengine_wrapper' => array(
				'type' => 'shipengine_tabs',
			),
			'gopremium'  => array(
				'type'          => 'activate_box',
				'class'                             => 'gopremium_tab_field',
			),
			
			'enabled'               => array(
				'title'             => __( 'Realtime Rates', 'wf-shipengine' ),
				'type'              => 'checkbox',
				'label'             => __( 'Enable', 'wf-shipengine' ),
				'description'       => __( 'Enable realtime rates on Cart/Checkout page.', 'wf-shipengine' ),
				'default'           => 'no',
				'desc_tip'          => true,
				'class'             => 'general_tab_field',
			),
			'title'                 => array(
				'title'             => __( 'Method Title', 'wf-shipengine' ),
				'type'              => 'text',
				'description'       => __( 'This controls the title for fall-back rate which the user sees during Cart/Checkout.', 'wf-shipengine' ),
				'default'           => __( $this->method_title, 'wf-shipengine' ),
				'placeholder'       => __( $this->method_title, 'wf-shipengine' ),
				'desc_tip'          => true,
				'class'             => 'rates_tab_field',
			),
			
			'availability' => array(
				'title' => __( 'Method Available to', 'wf-shipengine' ),
				'type' => 'select',
				'css'  => 'padding: 0px;',
				'default' => 'all',
				'description' => __( 'Select the countries', 'wf-shipengine' ),
				'desc_tip' => true,
				'class' => 'availability rates_tab_field',
				'options' => array(
					'all' => __( 'All Countries', 'wf-shipengine' ),
					'specific' => __( 'Specific Countries', 'wf-shipengine' ),
				),
			),
			'countries' => array(
				'title' => __( 'Specific Countries', 'wf-shipengine' ),
				'type' => 'multiselect',
				'class' => 'chosen_select rates_tab_field',
				'description' => __( 'Select specific countries', 'wf-shipengine' ),
				'desc_tip' => true,
				'css' => 'width: 450px;',
				'default' => '',
				'options' => $woocommerce->countries->get_allowed_countries(),
			),
			
			'debug_mode'            => array(
				'title'             => __( 'Debug Mode', 'wf-shipengine' ),
				'label'             => __( 'Enable', 'wf-shipengine' ),
				'type'              => 'checkbox',
				'default'           => 'no',
				'description'       => __( 'Enable debug mode to show debugging information on your cart/checkout. Not recommended to enable this in live site with traffic.', 'wf-shipengine' ),
				'desc_tip'          => true,
				'class'             => 'general_tab_field',
			),
			'api'                   => array(
				'title'             => __( 'Generic API Settings', 'wf-shipengine' ),
				'type'              => 'title',
				'description' => __( 'To obtain a ShipEngine.com API Key, Signup & Login to the ', 'wf-shipengine' ) . '<a href="http://www.shipengine.com?utm_source=elextensions" target="_blank">' . __( 'ShipEngine.com', 'wf-shipengine' ) . '</a>' . __( ' and then go to the ', 'wf-shipengine' ) . '<a href="https://www.shipengine.com/account/api-keys?utm_source=elextensions" target="_blank">' . __( 'API Keys section.', 'wf-shipengine' ) . '</a></br>' . __( 'You will find different API Keys for Live and Test mode.', 'wf-shipengine' ),
				'class'             => 'general_tab_field',
			),
			
			'api_mode'              => array(
				'title'             => __( 'API Mode', 'wf-shipengine' ),
				'type'              => 'select',
				'css'               => 'padding: 0px;',
				'default'           => 'Live',
				'options'           => $api_mode_options,
				'description'       => __( 'Live mode is the strict choice for Customers as Test mode is strictly restricted for development purpose by ShipEngine.com.', 'wf-shipengine' ),
				'desc_tip'          => true,
				'class'             => 'general_tab_field',
			),
			
			'api_key'               => array(
				'title'             => __( 'API-KEY', 'wf-shipengine' ),
				'type'              => 'password',
				'description'       => __( 'Live and Test mode APIs keys are different. Make sure to enter the right key based on API Mode.', 'wf-shipengine' ),
				'default'           => '',
				'desc_tip'          => true,
				'custom_attributes' => array(
					'autocomplete' => 'off',
				),
				'class'             => 'general_tab_field',
			),
			
		   
			'zip'                   => array(
				'title'             => __( 'Zip Code', 'wf-shipengine' ),
				'type'              => 'text',
				'description'       => __( 'Enter the postcode for the sender', 'wf-shipengine' ),
				'default'           => '',
				'desc_tip'          => true,
				'class'             => 'rates_tab_field',
			),
			'state' => array(
				'title' => __( 'Sender State Code', 'wf-shipengine' ),
				'type' => 'text',
				'description' => __( 'Enter state short code (Eg: CA).', 'wf-shipengine' ),
				'default' => '',
				'desc_tip' => true,
				'class' => 'rates_tab_field',
			),
			'country' => array(
				'title' => __( 'Sender Country', 'wf-shipengine' ),
				'type' => 'select',
				'class' => 'chosen_select',
				'css' => 'width: 450px;',
				'description' => __( 'Select the sender country', 'wf-shipengine' ),
				'desc_tip' => true,
				'default' => 'US',
				'options' => $woocommerce->countries->get_allowed_countries(),
				'class' => 'rates_tab_field',
			),
			'name' => array(
				'title' => __( 'Sender Name', 'elex-shipengine' ),
				'type' => 'text',
				'description' => __( 'Enter your name.', 'elex-shipengine' ),
				'default' => '',
				'desc_tip' => true,
				'class' => 'rates_tab_field',
				
			),
			'company' => array(
				'title' => __( 'Sender Company Name', 'elex-shipengine' ),
				'type' => 'text',
				'description' => __( 'Enter your company name.', 'elex-shipengine' ),
				'default' => '',
				'desc_tip' => true,
				'class' => 'rates_tab_field',
			),
			'street1' => array(
				'title' => __( 'Sender Address Line1', 'elex-shipengine' ),
				'type' => 'text',
				'description' => __( 'Enter your address line 1.', 'elex-shipengine' ),
				'default' => '',
				'desc_tip' => true,
				'class' => 'rates_tab_field',
			),
			'street2' => array(
				'title' => __( 'Sender Address Line2', 'elex-shipengine' ),
				'type' => 'text',
				'description' => __( 'Enter your address line 2.', 'elex-shipengine' ),
				'default' => '',
				'desc_tip' => true,
				'class' => 'rates_tab_field',
			),
			'city' => array(
				'title' => __( 'Sender City', 'elex-shipengine' ),
				'type' => 'text',
				'description' => __( 'Enter your city.', 'elex-shipengine' ),
				'default' => '',
				'desc_tip' => true,
				'class' => 'rates_tab_field',
			),
			
			'phone' => array(
				'title' => __( 'Sender Phone', 'elex-shipengine' ),
				'type' => 'text',
				'description' => __( 'Enter sender phone', 'elex-shipengine' ),
				'default' => '',
				'desc_tip' => true,
				'class' => 'rates_tab_field',
			),

			'selected_flat_rate_boxes' => array(
				'title' => __( 'Flat Rate Boxes <span style="vertical-align: super;color:green;font-size:12px">[Premium]</span>', 'wf-shipengine' ),
				'type' => 'multiselect',
				'class' => 'multiselect chosen_select selected_flat_rate_boxes rates_tab_field',
				'default' => '',
				'description' => __( 'Select flat rate boxes to make available.', 'wf-shipengine' ),
				'desc_tip' => false,
				'custom_attributes' => array( 'disabled' => 'disabled' ),
			),
			'flat_rate_fee' => array(
				'title' => __( 'Flat Rate Fee <span style="vertical-align: super;color:green;font-size:12px">[Premium]</span>', 'wf-shipengine' ),
				'type' => 'text',
				'description' => __( 'Fee per-box excluding tax. Enter an amount, e.g. 2.50, or a percentage, e.g. 5%. Leave blank to disable.', 'wf-shipengine' ),
				'default' => '',
				'desc_tip' => true,
				'class' => 'rates_tab_field',
				'custom_attributes' => array( 'disabled' => 'disabled' ),
			),
			
			'fallback'              => array(
				'title'             => __( 'Fallback', 'wf-shipengine' ),
				'type'              => 'text',
				'description'       => __( 'If ShipEngine.com returns no matching rates, offer this amount for shipping so that the user can still checkout. Leave blank to disable.', 'wf-shipengine' ),
				'default'           => '',
				'desc_tip'          => true,
				'class'             => 'rates_tab_field',
			),
			'show_rates' => array(
				'title' => __( 'Rates Type', 'wf-shipengine' ),
				'type' => 'select',
				'css'  => 'padding: 0px;',
				'default' => 'commercial',
				'options' => array(
					'residential' => __( 'Residential', 'wf-shipengine' ),
					'commercial' => __( 'Commercial', 'wf-shipengine' ),
				),
				'description' => __( 'Rates will be fetched based on the address type that you choose here. Please note this functionality will be available only for supported carriers.', 'wf-shipengine' ),
				'desc_tip' => true,
				'class' => 'rates_tab_field',
			),
			'shipengine_carrier'       => array(
				'title'           => __( 'ShipEngine Carrier(s)', 'wf-shipengine' ),
				'type'            => 'multiselect',
				'description'     => __( 'Select your ShipEngine Carriers.', 'wf-shipengine' ),
				'default'         => array( 'USPS' ),
				'css'             => 'width: 450px;',
				'class'           => 'ups_packaging chosen_select rates_tab_field',
				'options'         => $this->carrier_list,
				'desc_tip'        => true,
				'custom_attributes' => array(
					'autocomplete' => 'off',
				),
			),
			'shipengine_carrier_id' => array(
				'title' => __('ShipEngine Carrier Id(s)', 'elex-shipengine'),
				'type' => 'text',
				'description' => __('Enter your Carrier Ids or some carrier ids by "|" separating values.', 'elex-shipengine'),
				'default' => array('fedex'),
				'css' => 'width: 450px;',
				'placeholder' => __('Enter your Carrier Ids or some carrier ids by "|" separating values.', 'elex-shipengine'),
				'class' => 'ups_packaging rates_tab_field',
				'options' => $this->carrier_list,
				'desc_tip' => true,
				'custom_attributes' => array(
					'autocomplete' => 'off'
				),   
			),
			'services'              => array(
				'type'              => 'services',
				'class'             => 'rates_tab_field',
			),
		);
	}
   

	/**
	 * Calculate_shipping function.
	 *
	 * Access public
	 * Param mixed $package
	 * Return void
	 */
	public function calculate_shipping( $package = array() ) {
		global $woocommerce;
		$this->rates = array();
		$this->unpacked_item_costs = 0;
		$domestic = in_array( $package['destination']['country'], $this->domestic ) ? true : false;

		$this->debug( __( 'ShipEngine.com debug mode is on - to hide these messages, turn debug mode off in the settings.', 'wf-shipengine' ) );
		
		if ( $this->enable_standard_services ) {
			// Get cart package details and proceed with GetRates.
			$package_requests = $this->get_package_requests( $package );
			libxml_use_internal_errors( true );
			if ( $package_requests ) {

				$responses = array();
				foreach ( $package_requests as $key => $package_request ) {
					$responses[] = $this->get_result( $package_request );
				}
				if ( ! $responses ) {
					return false;
				}

				$found_rates = array();
				 $labels_stored = array();
				if ( is_array( $responses ) && ! empty( $responses ) ) {
					foreach ( $responses as $response_ele ) {
						$response_obj = ! empty( $response_ele['response'] ) ? $response_ele['response'] : array();
						$response_body = ! empty( $response_obj ) && ! is_object( $response_obj ) && isset( $response_obj['body'] ) ? json_decode( $response_obj['body'] ) : array();
						if ( ! empty( $response_body ) && isset( $response_body->rate_response ) && ! empty( $response_body->rate_response ) ) {
							$service_name_label = array();
							foreach ( $this->carrier as $carrier_name ) {
								$flag_currency_convertion = false;
								$responses_bodies = $response_body->rate_response;
								$responses_result   = isset( $responses_bodies->rates ) && ! empty( $responses_bodies->rates ) ? $responses_bodies->rates : ( isset( $responses_bodies->invalid_rates ) && ! empty( $responses_bodies->invalid_rates ) ? $responses_bodies->invalid_rates : array() ); 
								if ( ! empty( $responses_result ) ) {
									foreach ( $responses_result as $ship_engine_rate ) {
										if ( isset( $ship_engine_rate->carrier_code ) && $carrier_name == $ship_engine_rate->carrier_code ) {
											// Code snippet to use two different carrier accounts for domestic and international.
											$manage_accounts = apply_filters( 'elex_ship_engine_two_different_carrier_account', false );
											if ( $manage_accounts ) {
												if ( isset( $manage_accounts[ $carrier_name ] ) ) {
													if ( $domestic && isset( $manage_accounts[ $carrier_name ]['domestic'] ) ) {
														if ( $ship_engine_rate->carrier_account_id != $manage_accounts[ $carrier_name ]['domestic'] ) {
															continue;
														}
													}
													if ( ! $domestic && isset( $manage_accounts[ $carrier_name ]['international'] ) ) {
														if ( $ship_engine_rate->carrier_account_id != $manage_accounts[ $carrier_name ]['international'] ) {
															continue;
														}
													}
												}
											}
											if ( false == $flag_currency_convertion ) {
												 $shipping_amount = $ship_engine_rate->shipping_amount;
												$from_currency = isset( $shipping_amount->currency ) ? $shipping_amount->currency : '';
												$to_currency = get_woocommerce_currency();
												$converted_currency = $this->xa_currency_converter( $from_currency, $to_currency, $ship_engine_rate->service_code );
												$flag_currency_convertion = true;
											}
											if ( ! $converted_currency ) {
												break;
											}
										
											$service_type = (string) $ship_engine_rate->service_code;
											$service_name = '';
											if ( ! empty( $service_type ) ) {
												$service_name = (string) ( isset( $this->custom_services[ $carrier_name ][ $service_type ]['name'] ) && ! empty( $this->custom_services[ $carrier_name ][ $service_type ]['name'] ) ) ? $this->custom_services[ $carrier_name ][ $service_type ]['name'] : $this->services[ $carrier_name ]['services'][ $service_type ];
											}
										

										$shipping_amount = $ship_engine_rate->shipping_amount;
											$shipping_amount = isset( $shipping_amount->amount ) ? $shipping_amount->amount : 0;
											$total_amount = $response_ele['quantity'] * $shipping_amount;  

											//Added the other price which is charged by the carrier 
											if ( $ship_engine_rate->other_amount->amount ) {
												 $total_amount += $response_ele['quantity'] * $ship_engine_rate->other_amount->amount;
											}
										   //Added the confirmation price[Sigunature price] which is charged by the carrier
											if ( $ship_engine_rate->confirmation_amount->amount ) {
												 $total_amount += $response_ele['quantity'] * $ship_engine_rate->confirmation_amount->amount;
											}

										   //Added the Insurance price which is charged by the carrier
											if ( $ship_engine_rate->insurance_amount->amount ) {

												 $total_amount += $response_ele['quantity'] * $ship_engine_rate->insurance_amount->amount;
											}
										
										$total_amount = $total_amount * $converted_currency;
										// Sort

											if ( isset( $this->custom_services[ $carrier_name ][ $service_type ]['order'] ) ) {
												$sort = $this->custom_services[ $carrier_name ][ $service_type ]['order'];
											} else {
											$sort = 999;
											}

											if ( isset( $found_rates[ $service_type ] ) ) {
												$found_rates[ $service_type ]['cost'] = $found_rates[ $service_type ]['cost'] + $total_amount;
											} elseif ( empty( $labels_stored ) || in_array( $service_name, $labels_stored ) ) {
												$found_rates[ $service_type ]['label'] = $service_name;
												$found_rates[ $service_type ]['cost'] = $total_amount;
												$found_rates[ $service_type ]['carrier'] = $ship_engine_rate->carrier_code;
												$found_rates[ $service_type ]['sort'] = $sort;
												$found_rates[ $service_type ]['delivery_days'] = $ship_engine_rate->delivery_days;
											}
											if ( isset( $found_rates[ $service_type ]['label'] ) ) {
												$service_name_label []  = (string) $ship_engine_rate->service_code;
											}                                       
										}
									}
								}   
							}
						} else {
							$this->debug( __( 'ShipEngine.com - No rated returned from API.', 'elex-shipengine' ) );

							if ( $this->fallback ) {
								$this->debug( __( 'ShipEngine.com - Calculating fall back rates', 'elex-shipengine' ) );
								$rate = array(
									'id' => (string) $this->id . ':_fallback',
									'label' => (string) $this->title,
									'cost' => $this->fallback,
									'calc_tax' => 'per_item',
								);
								// Register the rate
								$this->add_rate( $rate );
							}
							// return;
						}
						
						if ( empty( $labels_stored ) ) {
							foreach ( $found_rates as $labels => $value ) {
								$labels_stored[] = $labels;
							} 
						}
			
						$check = false;
						foreach ( $labels_stored as $labels => $value ) { 
							foreach ( $service_name_label as $service => $value_service ) {
								if ( $value == $value_service ) {
									$check = true;
								}
							}
							if ( true != $check ) {
								unset( $found_rates[ $value ] );
							}
							$check = false;
						}
					}
				}
				$rate_added = 0;
				if ( $found_rates ) {
					uasort( $found_rates, array( $this, 'sort_rates' ) );
					foreach ( $this->carrier as $carrier_name ) {
						foreach ( $found_rates as $service_type => $found_rate ) {
							// Enabled check
							$insurance_amount = 0;
							if ( $carrier_name == $found_rate['carrier'] ) {
								if ( isset( $this->custom_services[ $carrier_name ][ $service_type ] ) && empty( $this->custom_services[ $carrier_name ][ $service_type ]['enabled'] ) ) {
									continue;
								}
								$total_amount = $found_rate['cost'];
								$delivery = $found_rate['delivery_days'];
								// Cost adjustment %
								if ( ! empty( $this->custom_services[ $carrier_name ][ $service_type ]['adjustment_percent'] ) ) {
									$total_amount = $total_amount + ( $total_amount * ( floatval( $this->custom_services[ $carrier_name ][ $service_type ]['adjustment_percent'] ) / 100 ) );
								}
								// Cost adjustment
								if ( ! empty( $this->custom_services[ $carrier_name ][ $service_type ]['adjustment'] ) ) {
									$total_amount = $total_amount + floatval( $this->custom_services[ $carrier_name ][ $service_type ]['adjustment'] );
								}
								if ( $service_type ) {
									$labelName = ! empty( $this->settings['services'][ $carrier_name ][ $service_type ]['name'] ) ? $this->settings['services'][ $carrier_name ][ $service_type ]['name'] : $this->services[ $carrier_name ]['services'][ $service_type ]; 
									$rate = array(
										'id' => (string) $this->id . ':' . $service_type,
										'label' => (string) $labelName,
										'cost' => (string) $total_amount,
										'meta_data' => array( 'shipengine_delivery_time' => $delivery ),
										'calc_tax' => 'per_order',
									);
								// Register the rate
								$this->add_rate( $rate );
								$rate_added++;
								}                           
							}
						}
					}
				}
			}
		}

	}
	
	public function xa_currency_converter( $from_currency, $to_currency, $carrier ) {
		if ( $from_currency == $to_currency ) {
			 return 1;
		} else {
			$from_currency = urlencode( $from_currency );
			$to_currency = urlencode( $to_currency );
			try {
			$result = wp_safe_remote_get( "https://www.alphavantage.co/query?function=CURRENCY_EXCHANGE_RATE&from_currency=$from_currency&to_currency=$to_currency&apikey=G1QF4V7WM07HNOB2" );
				if ( false === $result ) {
				   throw new Exception( "Unable to receive currency conversion response from Google Finance API call ( https://finance.google.com/finance/converter ). Skipping the shipping rates for the carrier $carrier as shop currency and the currency returned by Rates API differs." );
				}
			} catch ( Exception $e ) {
				 $this->debug( __( $e->getMessage(), 'wf-shipengine' ) );
				 return 0;
			}
			if ( isset( $result['body'] ) && ! is_object( $result['body'] ) ) {
			 $result = json_decode( $result['body'], true );   
			} else {
				$result = array();
			}
			
			$converted_currency = isset( $result['Realtime Currency Exchange Rate']['5. Exchange Rate'] ) ? $result['Realtime Currency Exchange Rate']['5. Exchange Rate'] : 1 ;
			return $converted_currency;
		}
	}

	/**
	*Function to check the address fields.
	*/
	public function elex_check_address_fields( $package ) {

		//check 
		if ( ! isset( $package['request'] ) ) {
			return false;
		}
		
		//address check
		if ( isset( $package['request'] ) ) {
			if ( '' != $package['request']['Rate']['ToZIPCode'] && '' != $package['request']['Rate']['ToCountry'] ) {
				return true;
			} else {
				$this->debug( __( 'To process the shipping rates destination country and postcode are mandatory', 'wf-shipengine' ) );
				return false;
			}
		}

	}

	public function elex_get_carrier_ids( $ids ) {
		$ids = str_replace(' ', '', $ids);
		$shippment_ids = explode( '|' , $ids);
		return $shippment_ids;
	}

	private function get_result( $package_request, $predefined_package = '' ) {
		// Get rates.
		$address_check = $this->elex_check_address_fields( $package_request );
		$carrier_ids    = $this->elex_get_carrier_ids($this->carrier_id);


		if ( $address_check ) {
			$carrier_ids = json_encode($carrier_ids);
			try {
				$payload = array();
				//Multi-vendor Support
				 $payload = '
                             {
                                "rate_options": { "carrier_ids":' . $carrier_ids . '},
                                "shipment":{
                                    
                                    "ship_to":{
                                        "postal_code":"' . $package_request['request']['Rate']['ToZIPCode'] . '",
                                        "country_code": "' . $package_request['request']['Rate']['ToCountry'] . '",
                                        "address_residential_indicator": "' . $this->resid . '",
                                    },
                                    "ship_from":{
                                ';



			 
				$payload .= '
                                "company_name": "' . $this->senderCompanyName . '",
                                "name": "' . $this->senderName . '",
                                "phone": "' . $this->senderPhone . '",
                                "address_line1": "' . $this->senderAddressLine1 . '",
                                "address_line2": "' . $this->senderAddressLine2 . '",
                                "city_locality": "' . $this->senderCity . '",
                                "state_province": "' . $this->senderState . '",
                                "postal_code": "' . $this->zip . '",
                                "country_code": "' . $this->senderCountry . '",

                            },';
				

				if ( ! empty( $package_request['request']['Rate']['WeightLb'] ) && 0.00 == $package_request['request']['Rate']['WeightOz'] ) {
					$package_request['request']['Rate']['WeightOz'] = number_format( $package_request['request']['Rate']['WeightLb'] * 16, 0, '.', '' );
				}
				

			   
			$payload .= '
                            "confirmation":"none",';
		   
			$payload .= '
                             "packages":[
                                    {
                                
                                "weight":{
                                    "value": "' . $package_request['request']['Rate']['WeightOz'] . '",
                                    "unit": "ounce"
                                },';
			
			$payload .= '       
                    "dimensions": {
                        "unit": "inch",
                        "length": "' . $package_request['request']['Rate']['Length'] . '",
                        "width": "' . $package_request['request']['Rate']['Width'] . '",
                        "height": "' . $package_request['request']['Rate']['Height'] . '"
                    },';
							 
			$payload .= ' 
                            }
                        ]
                    }
                }';                
			   $requestHeaders = array(
				   'Api-Key' => $this->api_key,
				   'Host' => ' api.shipengine.com',
				   'Content-Type' => 'application/json',
			   );
			   $this->debug( 'ShipEngine REQUEST API: <pre>' . print_r( $requestHeaders, true ) . '</pre>' );

				$this->debug( 'ShipEngine REQUEST: <pre>' . print_r( $payload, true ) . '</pre>' );

				$response = wp_safe_remote_post(
					$this->rates_endpoint, 
					array(
						'method' => 'POST',
						'headers' => $requestHeaders,
						'timeout' => 70,
						'body' => $payload,
					)
				);
				$this->debug( 'ShipEngine: <pre>' . print_r( $response, true ) . '</pre>' );
				$response_ele = array();
				$response_ele['response'] = $response;
				$response_ele['quantity'] = $package_request['quantity'];


			} catch ( Exception $e ) {
				
				if ( ! empty( $e->getMessage() ) ) {
					if ( $this->fallback ) {
				$this->debug( __( 'ShipEngine.com - Calculating fall back rates', 'elex-shipengine' ) );
				$rate = array(
					'id' => (string) $this->id . ':_fallback',
					'label' => (string) $this->title,
					'cost' => $this->fallback,
					'calc_tax' => 'per_item',
				);
				// Register the rate
				$this->add_rate( $rate );
					}
				}
				
				$this->debug( __( 'ShipEngine.com - Unable to Get Rates: ', 'elex-shipengine' ) . $e->getMessage() );
				if ( ELEX_SHIPENGINE_ADV_DEBUG_MODE == 'on' ) {
					$this->debug( print_r( $e, true ) );
				}
				return false;
			}
	
			return $response_ele;
		} else {
			return false;
		}
	}

	/**
	 * Prepare_rate function.
	 *
	 * Access private
	 * Param mixed $rate_code
	 * Param mixed $rate_id
	 * Param mixed $rate_name
	 * Param mixed $rate_cost
	 * Return void
	 */
	private function prepare_rate( $rate_code, $rate_id, $rate_name, $rate_cost ) {

		// Name adjustment
		if ( ! empty( $this->custom_services[ $rate_code ]['name'] ) ) {
			$rate_name = $this->custom_services[ $rate_code ]['name'];
		}

		// Merging
		if ( isset( $this->found_rates[ $rate_id ] ) ) {
			$rate_cost = $rate_cost + $this->found_rates[ $rate_id ]['cost'];
			$packages = 1 + $this->found_rates[ $rate_id ]['packages'];
		} else {
			$packages = 1;
		}

		// Sort
		if ( isset( $this->custom_services[ $rate_code ]['order'] ) ) {
			$sort = $this->custom_services[ $rate_code ]['order'];
		} else {
			$sort = 999;
		}

		$this->found_rates[ $rate_id ] = array(
			'id'        => $rate_id,
			'label'     => $rate_name,
			'cost'      => $rate_cost,
			'sort'      => $sort,
			'packages'  => $packages,
		);
	}

	/**
	 * Sort rates function.
	 *
	 * Access public
	 * Param mixed $a
	 * Param mixed $b
	 * Return void
	 */
	public function sort_rates( $a, $b ) {
		if ( $a['sort'] == $b['sort'] ) {
			return 0;
		}
		return ( $a['sort'] < $b['sort'] ) ? -1 : 1;
	}

	/**
	 * Get_request function.
	 *
	 * Access private
	 * Return void
	 */
	// WF - Changing function to public.
	public function get_package_requests( $package ) {
		$requests = $this->per_item_shipping( $package );

		return $requests;
	}

	/**
	 * Per_item_shipping function.
	 *
	 * Access private
	 * Param mixed $package
	 * Return void
	 */
	private function per_item_shipping( $package ) {
		global $woocommerce;
	   
		$requests = array();    
		$domestic = in_array( $package['destination']['country'], $this->domestic ) ? true : false;

		// Get weight of order
		foreach ( $package['contents'] as $item_id => $values ) {
			$values['data'] = $this->wf_load_product( $values['data'] );

			if ( ! $values['data']->needs_shipping() ) {
				$this->debug( sprintf( __( 'Product # is virtual. Skipping.', 'wf-shipengine' ), $item_id ) );
				continue;
			}

			if ( ! $values['data']->get_weight() ) {
				$this->debug( sprintf( __( 'Product # is missing weight. Using 1lb.', 'wf-shipengine' ), $item_id ) );

				$weight = 1;
				$weightoz = 1; // added for default
			} else {
				$weight = wc_get_weight( $values['data']->get_weight(), 'lbs' );
				$weightoz = wc_get_weight( $values['data']->get_weight(), 'oz' );
			}

			$size = 'REGULAR';

			if ( $values['data']->length && $values['data']->height && $values['data']->width ) {

				$dimensions = array( wc_get_dimension( $values['data']->length, 'in' ), wc_get_dimension( $values['data']->height, 'in' ), wc_get_dimension( $values['data']->width, 'in' ) );

				sort( $dimensions );

				if ( max( $dimensions ) > 12 ) {
					$size = 'LARGE';
				}

				$girth = $dimensions[0] + $dimensions[0] + $dimensions[1] + $dimensions[1];
			} else {
				$dimensions = array( 0, 0, 0 );
				$girth = 0;
			}

			$quantity = $values['quantity'];

			if ( 'LARGE' === $size ) {
				$rectangular_shaped = 'true';
			} else {
				$rectangular_shaped = 'false';
			}

			$dest_postal_code = ! empty( $package['destination']['postcode'] ) ? $package['destination']['postcode'] : ( isset( $package['destination']['zip'] ) ? $package['destination']['zip'] : '' );
			if ( $domestic ) {
				$request['Rate'] = array(
					'FromZIPCode'       => str_replace( ' ', '', strtoupper( $this->settings['zip'] ) ),
					'ToZIPCode'         => strtoupper( substr( $dest_postal_code, 0, 5 ) ),
					'ToCountry'         => $package['destination']['country'],
					'WeightLb'          => floor( $weight ),
					'WeightOz'          => number_format( $weightoz, 0, '.', '' ),
					'PackageType'       => 'Package',
					'Length'            => $dimensions[2],
					'Width'             => $dimensions[1],
					'Height'            => $dimensions[0],
					'ShipDate'          => gmdate( 'Y-m-d', ( current_time( 'timestamp' ) + ( 60 * 60 * 24 ) ) ),
					'InsuredValue'      => $values['data']->get_price(),
					'RectangularShaped' => $rectangular_shaped,
				);
			} else {
				$request['Rate'] = array(
					'FromZIPCode'       => str_replace( ' ', '', strtoupper( $this->settings['zip'] ) ),
					'ToZIPCode'         => $dest_postal_code,
					'ToCountry'         => $package['destination']['country'],
					'Amount'            => $values['data']->get_price(),
					'WeightLb'          => floor( $weight ),
					'WeightOz'          => number_format( $weightoz, 0, '.', '' ),
					'PackageType'       => 'Package',
					'Length'            => $dimensions[2],
					'Width'             => $dimensions[1],
					'Height'            => $dimensions[0],
					'ShipDate'          => gmdate( 'Y-m-d', ( current_time( 'timestamp' ) + ( 60 * 60 * 24 ) ) ),
					'InsuredValue'      => $values['data']->get_price(),
					'RectangularShaped' => $rectangular_shaped,
				);
			}
			
			$request['unpacked']    = array();
			$request['packed']      = array( $values['data'] );

			$request_ele                = array();
			$request_ele['request']     = $request;
			$request_ele['quantity']    = $quantity;

			$requests[]                 = $request_ele;
		}
		return $requests;
	}

	/**
	 * Box_shipping function.
	 *
	 * Access private
	 * Param mixed $package
	 * Return void
	 */

	public function debug( $message, $type = 'notice' ) {
		if ( $this->debug && ! is_admin() ) { //WF: is_admin check added.
			if ( version_compare( WOOCOMMERCE_VERSION, '2.1', '>=' ) ) {
				wc_add_notice( $message, $type );
			} else {
				global $woocommerce;
				$woocommerce->add_message( $message );
			}
		}
	}
	
	
	private function wf_load_product( $product ) {
		if ( ! $product ) {
			return false;
		}
		if ( ! class_exists( 'ElexProduct' ) ) {
			include_once( 'class-wf-legacy.php' );
		}
		if ( $product instanceof ElexProduct ) {
			return $product;
		}
		return ( WC()->version < '2.7.0' ) ? $product : new ElexProduct( $product );
	}

}
