<?php
/*
	Plugin Name: ELEX ShipEngine UPS & FedEx Shipping Method (BASIC)
	Plugin URI: https://elextensions.com/plugin/elex-shipengine-multi-carrier-shipping-label-printing-plugin-for-woocommerce/
	Description: Using ShipEngine shipping APIs obtain UPS and FedEx real time shipping rates.
	Version: 1.0.9
	WC requires at least: 2.6.0
	WC tested up to: 7.5
	Author: ELEXtensions
	Author URI: https://elextensions.com/plugin/elex-shipengine-multi-carrier-shipping-plugin-for-woocommerce-free-version/
*/

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

// WC active check
if ( ! ( in_array( 'woocommerce/woocommerce.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) ) ) {
		add_action( 'admin_notices', 'xa_basic_shipengine_woocommerce_inactive_notice' );
		return;
}

function xa_basic_shipengine_woocommerce_inactive_notice() {
	?>
<div id="message" class="error">
	<p>
	<?php	printf( esc_html_e( '<b>WooCommerce</b> plugin must be active for <b>ShipEngine WooCommerce Extension (BASIC)</b> to work. ', 'eh_shipengine' ) ); ?>
	</p>
</div>
<?php
}

if ( ! defined( 'ELEX_SHIPENGINE_ID' ) ) {
	define( 'ELEX_SHIPENGINE_ID', 'elex_shipengine_id' );
}

if ( ! defined( 'ELEX_SHIPENGINE_VERSION' ) ) {
	define( 'ELEX_SHIPENGINE_VERSION', '1.0.0' );
}

if ( ! defined( 'ELEX_SHIPENGINE_ADV_DEBUG_MODE' ) ) {
	define( 'ELEX_SHIPENGINE_ADV_DEBUG_MODE', 'on' ); // Turn "off" to disable advanced logs.
}

function wf_shipengine_basic_activation_check() {
	if ( is_plugin_active( 'shipengine-woocommerce-shipping/multicarrier-woocommerce-shipping.php' ) ) {
		deactivate_plugins( basename( __FILE__ ) );
		wp_die( wp_kses("Oops! You tried installing the basic version without deactivating and deleting the premium version. Kindly deactivate and delete the premium version and then try again. For any issues, raise a ticket via <a target='_blank' href='https://elextensions.com/support/'>Support</a>", 'eh_shipengine' ), '', array( 'back_link' => 1 ) );
	}

	if ( ! function_exists( 'curl_init' ) ) {
		deactivate_plugins( basename( __FILE__ ) );
		wp_die( esc_html_e( 'ShipEngine needs the CURL PHP extension.', 'eh_shipengine' ) );
	}

	if ( ! function_exists( 'json_decode' ) ) {
		deactivate_plugins( basename( __FILE__ ) );
		wp_die( esc_html_e( 'ShipEngine needs the JSON PHP extension.', 'eh_shipengine' ) );
	}
}
register_activation_hook( __FILE__, 'wf_shipengine_basic_activation_check' );

/**
 * WC_USPS class
 */
if ( ! class_exists( 'ElexShipEngineWooCommerceShipping' ) ) {
	class ElexShipEngineWooCommerceShipping {

		/**
		 * Constructor
		 */
		public function __construct() {
			add_action( 'init', array( $this, 'init' ) );
			add_filter( 'plugin_action_links_' . plugin_basename( __FILE__ ), array( $this, 'plugin_action_links' ) );
			add_action( 'woocommerce_shipping_init', array( $this, 'shipping_init' ) );
			add_filter( 'woocommerce_shipping_methods', array( $this, 'add_method' ) );
			add_action( 'admin_enqueue_scripts', array( $this, 'scripts' ) );
		}

		/**
		 * Localisation
		 */
		public function init() {
			if ( ! class_exists( 'ElexOrder' ) ) {
				include_once 'includes/class-wf-legacy.php';
			}
			load_plugin_textdomain( 'eh_shipengine', false, dirname( plugin_basename( __FILE__ ) ) . '/languages/' );
		}

		/**
		 * Plugin page links
		 */
		public function plugin_action_links( $links ) {
			$plugin_links = array(
				'<a href="' . admin_url( 'admin.php?page=wc-settings&tab=shipping&section=elex_shipengine_id' ) . '">' . __( 'Settings', 'eh_shipengine' ) . '</a>',

				'<a href="https://elextensions.com/support/" target="_blank">' . __( 'Support', 'eh_shipengine' ) . '</a>',

				'<a href="https://elextensions.com/plugin/elex-shipengine-multi-carrier-shipping-label-printing-plugin-for-woocommerce/" target="_blank">' . __( 'Premium Upgrade', 'eh_shipengine' ) . '</a>',

			);
			return array_merge( $plugin_links, $links );
		}

		/**
		 * Load gateway class
		 */
		public function shipping_init() {
			include_once( 'includes/class-wf-shipping-shipengine.php' );
		}

		/**
		 * Add method to WC
		 */
		public function add_method( $methods ) {
			$methods[] = 'ElexShipEngine';
			return $methods;
		}

		/**
		 * Enqueue scripts
		 */
		public function scripts() {
			wp_enqueue_script( 'jquery' );
			wp_enqueue_script( 'jquery-ui-sortable' );
			wp_enqueue_style( 'wf-common-style', plugins_url( '/resources/css/wf_common_style.css', __FILE__ ), array(), ELEX_SHIPENGINE_VERSION );
			wp_enqueue_style( 'wf-bootstrap', plugins_url( '/resources/css/bootstrap.css', __FILE__ ), array(), ELEX_SHIPENGINE_VERSION );
			wp_enqueue_script( 'wf-common-script', plugins_url( '/resources/js/wf_common.js', __FILE__ ), array( 'jquery' ), ELEX_SHIPENGINE_VERSION );
			wp_enqueue_script( 'eh_shipengine-script', plugins_url( '/resources/js/elex_shipengine.js', __FILE__ ), array( 'jquery' ), ELEX_SHIPENGINE_VERSION );

		}
	}
	new ElexShipEngineWooCommerceShipping();
	// review component
	if ( ! function_exists( 'get_plugin_data' ) ) {
		require_once  ABSPATH . 'wp-admin/includes/plugin.php';
	}
	include_once __DIR__ . '/review_and_troubleshoot_notify/review-and-troubleshoot-notify-class.php';
	$data                      = get_plugin_data( __FILE__ );
	$data['name']              = $data['Name'];
	$data['basename']          = plugin_basename( __FILE__ );
	$data['rating_url']        = 'https://elextensions.com/plugin/elex-shipengine-multi-carrier-shipping-plugin-for-woocommerce-free-version/#reviews';
	$data['documentation_url'] = 'https://elextensions.com/knowledge-base/set-up-elex-shipengine-multi-carrier-shipping-label-printing-plugin-for-woocommerce/';
	$data['support_url']       = 'https://wordpress.org/support/plugin/elex-shipengine-shipping-method/';

	new \Elex_Review_Components( $data );
}
